*************************************************************************************
*														*
*	Pseudo random number generator for the EASy68k simulator	2006/01/10		*
*														*
*	This code is an attempt to gauge the effectiveness of variations on the		*
*	shift register type of pseudo random number generator. The PRNG code is		*
*	similar to that used in the RND() function in EhBASIC68.				*
*														*
*	More 68000 and other projects can be found on my website at ..			*
*														*
*	 http://mycorner.no-ip.org/index.html							*
*														*
*	mail : leeedavison@googlemail.com								*
*														*
*************************************************************************************

*************************************************************************************
* This program opens a file and, assuming the file opened correctly, repeatedly calls
* a function to generate the next number in a pseudo random sequence. A byte from
* this number is output to the file after each call and after 65536 bytes have been
* written the file is closed and the program stops. This file can then be analysed to
* gauge the randomness of this relatively short sequence.

	ORG		$1000				* set origin

start
	LEA		m_init(pc),a1		* get pointer to message
	MOVE		#13,d0			* output string with [CR][LF]
	TRAP		#15				* do I/O function

	LEA		(LAB_VARS,PC),a3		* get pointer to variables
	MOVE.l	#$8C9F53D0,PRNlword(a3)	* set PRNG seed. this can be any value
							* except zero, changing this will produce
							* a different but repeatable set of bytes

* start by opening a file for the result bytes

	LEA		m_open(pc),a1		* get pointer to message
	MOVE		#13,d0			* output string with [CR][LF]
	TRAP		#15				* do I/O function

	LEA		(LAB_FLNM,PC),a1		* get pointer to filename
	MOVE		#52,d0			* open new file
	TRAP		#15				* do I/O function

	TST.w		d0				* test save result
	BNE		file_exit			* if error clear up and exit

* file is open

	MOVE.l	d1,file_id(a3)		* save the file ID

* now repeatedly call the code and output the bytes to the file

	LEA		m_write(pc),a1		* get pointer to message
	MOVE		#13,d0			* output string with [CR][LF]
	TRAP		#15				* do I/O function

	MOVE.w	#$FFFF,d4			* set for $10000 loops
LAB_TSTL
	MOVE.w	#$00FF,d2			* set mask
	AND.w		d4,d2				* mask count
	BNE.s		no_dot			* don't print '.' if not 256th byte

	MOVEQ		#'.',d1			* set "." character
	MOVE		#6,d0				* output character
	TRAP		#15				* do I/O function

	MOVE.w	#$1F00,d2			* set mask
	AND.w		d4,d2				* mask count
	BNE.s		no_dot			* don't print [CR] if not 32nd dot

	MOVEQ		#$0D,d1			* set "." character
	TRAP		#15				* do I/O function
	MOVEQ		#$0A,d1			* set "." character
	TRAP		#15				* do I/O function

no_dot
	BSR		LAB_RND			* call the PRNG code
	MOVE.b	PRNlword(a3),d0		* get byte from result
	BSR		byte_out			* output it to file
	DBF		d4,LAB_TSTL			* loop until done

* either the file failed to open or is done so now close and exit

file_exit
	LEA		m_close(pc),a1		* get pointer to message
	MOVE		#13,d0			* output string with [CR][LF]
	TRAP		#15				* do I/O function

	MOVEQ		#50,d0			* close all files
	TRAP		#15				* do I/O function

	LEA		m_done(pc),a1		* get pointer to message
	MOVE		#13,d0			* output string with [CR][LF]
	TRAP		#15				* do I/O function

	MOVEQ		#9,d0				* terminate program
	TRAP		#15				* do I/O function


*************************************************************************************
* output character to file from register d0

byte_out
	MOVEM.l	d0-d2/a1,-(sp)		* save d0, d1, d2 and a1
	MOVE.l	file_id(a3),d1		* get the file ID
	LEA		file_byte(a3),a1		* point to the byte buffer
	MOVE.b	d0,(a1)			* save byte
	MOVEQ		#1,d2				* set byte count
	MOVEQ		#54,d0			* set write to file
	TRAP		#15				* do I/O function
	MOVEM.l	(sp)+,d0-d2/a1		* restore d0, d1, d2 and a1
	RTS

*************************************************************************************
* This is the code that generates the pseudo random sequence. A seed word located in
* PRNlword(a3) is loaded into a register before being operated on to generate the
* next number in the sequence. This number is then saved as the seed for the next
* time it's called.
*
* This code is adapted from the 32 bit version of RND(n) used in EhBASIC68. Taking
* the 19th next number is slower but helps to hide the shift and add nature of this
* generator as can be seen from analysing the output.

LAB_RND
	MOVE.l	PRNlword(a3),d0		* get current seed longword
	MOVEQ		#$AF-$100,d1		* set EOR value
	MOVEQ		#18,d2			* do this 19 times
Ninc0
	ADD.l		d0,d0				* shift left 1 bit
	BCC.s		Ninc1				* if bit not set skip feedback

	EOR.b		d1,d0				* do Galois LFSR feedback
Ninc1
	DBF		d2,Ninc0			* loop

	MOVE.l	d0,PRNlword(a3)		* save back to seed longword

	RTS

*************************************************************************************
* output file name

LAB_FLNM
	dc.b	'output.bin',$00			* filename
m_init
	dc.b	'Initializing',$00
m_open
	dc.b	'Opening file',$00
m_write
	dc.b	'Writing bytes',$00
m_close
	dc.b	'Closing file',$00
m_done
	dc.b	'Done',$00

	ds.w	0					* ensure even address

*************************************************************************************
* variables used

LAB_VARS

	OFFSET	0				* going to use relative addressing

PRNlword	ds.l	1				* PRNG seed long word

file_id	ds.l	1				* file id store
file_byte	ds.w	1				* byte to send to file

	ORG	*

	END	start

*************************************************************************************
      "The generation of random numbers is too important to be left to chance."
                                  Robert R.Coveyou.
*************************************************************************************

